<?php

if (!defined('IN_KANCART')) {
    header('HTTP/1.1 404 Not Found');
    die();
}

class CategoryService {

    public function getAllCategories() {
        global $languages_id;
        $sql = 'SELECT
                    c.*, cd.categories_name
                FROM categories AS c
                LEFT JOIN categories_description AS cd ON c.categories_id = cd.categories_id
                WHERE cd.language_id = ' . intval($languages_id) . ' order by c.sort_order,cd.categories_name';
        $query = tep_db_query($sql);
        $pos = 0;
        $categories = array();
        while ($row = tep_db_fetch_array($query)) {
            $categories[] = array(
                'cid' => $row['categories_id'],
                'parent_cid' => $row['parent_id'] == 0 ? '-1' : $row['parent_id'],
                'name' => $row['categories_name'],
                'position' => $pos++
            );
        }
        // the order must be 'addIsParentFieldToCategories and then addCountFieldToCategories'
        $this->addIsParentFieldToCategories($categories);
        $this->addCountFieldToCategories($categories);
        return $categories;
    }

    public function addIsParentFieldToCategories(&$categories) {
        foreach ($categories as &$category) {
            if ($this->hasSubCategory($categories, $category['cid'])) {
                $category['is_parent'] = true;
            } else {
                $category['is_parent'] = false;
            }
        }
    }

    private function addCountFieldToCategories(&$categories) {
        foreach ($categories as &$category) {
            $subCategories = $this->getSubCategories($category['cid'], $categories);
            $category['count'] = $this->getProductQuantity($category['cid'], $subCategories);
        }
    }

    private function getProductQuantity($categoryId, $subCategories) {
        $ids = array();
        $ids[] = $categoryId;
        foreach ($subCategories as $category) {
            $ids[] = intval($category['cid']);
        }
        $sql = 'SELECT COUNT(*) as count FROM ' . TABLE_PRODUCTS_TO_CATEGORIES . ' where categories_id in (' . join(',', $ids) . ')';
        $query = tep_db_query($sql);
        $countResult = tep_db_fetch_array($query);
        return $countResult['count'];
    }

    public function hasSubCategory($allCategories, $parentId) {
        foreach ($allCategories as $cat) {
            if ($cat['parent_cid'] == $parentId) {
                return true;
            }
        }
        return false;
    }

    public function getSubCategories($parentId, $allCategories) {
        $subs = array();
        $this->getSubCategoriesRecursively($parentId, $allCategories, $subs);
        return $subs;
    }

    public function getSubCategoriesIds($parentId, $allCategories) {
        $subs = $this->getSubCategories($parentId, $allCategories);
        $ids = array();
        foreach ($subs as $sub) {
            $ids[] = $sub['cid'];
        }
        return $ids;
    }

    private function getSubCategoriesRecursively($parentId, $allCategories, &$subs) {
        foreach ($allCategories as $category) {
            if ($category['parent_cid'] == $parentId) {
                $subs[] = $category;
                if ($category['is_parent']) {
                    $this->getSubCategoriesRecursively($category['cid'], $allCategories, $subs);
                }
            }
        }
    }

    public function getProductCount($ids) {
        $idsSql = join(',', $ids);
        $categoryFilterSql = count($ids) > 1 ? "p2c.categories_id in ($idsSql)" : "p2c.categories_id = $idsSql";
        $sql = "SELECT
            COUNT(*) as count
            FROM
                products p,products_to_categories p2c
            WHERE
                p.products_status = 1
            AND p.products_id = p2c.products_id
            AND $categoryFilterSql";
        $query = tep_db_query($sql);
        $count = tep_db_fetch_array($query);
        return $count['count'];
    }

    /**
     * get products by category,the category id is specified in the $filter array
     * @global type $languages_id
     * @param type $filter
     * @return type
     */
    public function getProducts($filter) {
        global $languages_id;
        $categoryId = $filter['cid'];
        $allCategories = $this->getAllCategories();
        $ids = array($categoryId == -1 ? 0 : $categoryId);
        $subCategoryIds = $this->getSubCategoriesIds($categoryId, $allCategories);
        $allIds = array_merge($ids, $subCategoryIds);
        $idsSql = join(',', $allIds);
        $filterCategorySql = count($allIds) > 1 ? "p2c.categories_id in ($idsSql)" : "p2c.categories_id = $idsSql";
        $totalResults = $this->getProductCount($allIds);
        $pageNo = ($filter['page_no'] - 1 < 0 ? 0 : $filter['page_no'] - 1) * $filter['page_size'];
        $proudctTranslator = ServiceFactory::factory('ProductTranslator');
        $sql = "SELECT
            p.*,
            pd.*,
            m.*,
            IF (
                s.STATUS,
                s.specials_new_products_price,
                NULL
            ) AS specials_new_products_price,
            IF (
               s.STATUS,
               s.specials_new_products_price,
               p.products_price
            ) AS final_price
            FROM
                products_description pd,
                products p
            LEFT JOIN manufacturers m ON p.manufacturers_id = m.manufacturers_id
            LEFT JOIN specials s ON p.products_id = s.products_id,
             products_to_categories p2c
            WHERE
                    p.products_status = '1'
            AND p.products_id = p2c.products_id
            AND pd.products_id = p2c.products_id
            AND pd.language_id = $languages_id
            AND $filterCategorySql
            ORDER BY {$filter['sort_by']} {$filter['sort_order']}
            LIMIT $pageNo,{$filter['page_size']}";
        $query = tep_db_query($sql);
        $products = array('total_results' => $totalResults);
        $products['items'] = array();
        while ($row = tep_db_fetch_array($query)) {
            $proudctTranslator->clear();
            $proudctTranslator->setProduct($row);
            $proudctTranslator->getItemBaseInfo();
            $proudctTranslator->getItemPrices();
            $products['items'][] = $proudctTranslator->getTranslatedItem();
        }
        return $products;
    }

}

?>
