<?php

if (!defined('IN_KANCART')) {
    header('HTTP/1.1 404 Not Found');
    die();
}

/**
 * Checkout utility
 * @package services
 * @author hujs
 */
class CheckoutService {

    public function __construct() {
        $this->cart = &$_SESSION['cart'];
        $this->freeShipping = false;
        $this->isVirtual = $this->cart->get_content_type() == 'virtual';
    }

    /**
     * get checkout detail information
     * 
     */
    public function detail() {
        $detail = array();
        $detail['billing_address'] = $this->getBillingAddress();
        $detail['shipping_address'] = $this->getShippingAddress();
//        $detail['need_shipping_address'] = !$this->checkAddressIntergrity($detail['shipping_address']);
//        $detail['need_billing_address'] = !$this->checkAddressIntergrity($detail['shipping_address']);
        $detail['review_orders'] = array($this->getReviewOrder());
        if (!$this->freeShipping && !$detail['review_orders'][0]['selected_shipping_method_id']) {
            $detail['need_select_shipping_method'] = true;
        }
        $detail['price_infos'] = $this->getPriceInfos();
        $detail['payment_methods'] = $this->getPaymentMethods();
        $detail['is_virtual'] = $this->isVirtual;

        return $detail;
    }

    public function getReviewOrder() {
        $order = array();
        $order['cart_items'] = $this->getorderItems();
        $order['shipping_methods'] = $this->getOrderShippingMethods();
        $selectedShippingMethod = $this->selectShippingMethod();
        $order['selected_shipping_method_id'] = $this->freeShipping ? 0 : $selectedShippingMethod;
        return $order;
    }

    public function checkAddressIntergrity($address) {
        if ($address) {
            $userService = ServiceFactory::factory('User');
            return $userService->checkAddressIntegrity($address);
        }
        return false;
    }

    public function buildWpsRedirectParams($paypalStandard, $returnUrl = '', $cancelUrl = '') {
        global $customer_id, $order, $sendto, $currency, $cart_PayPal_Standard_ID;
        $process_button_string = '';
        $parameters = array(
            'cmd' => '_xclick',
            'item_name' => STORE_NAME,
            'shipping' => $paypalStandard->format_raw($order->info['shipping_cost']),
            'tax' => $paypalStandard->format_raw($order->info['tax']),
            'business' => MODULE_PAYMENT_PAYPAL_STANDARD_ID,
            'amount' => $paypalStandard->format_raw($order->info['total'] - $order->info['shipping_cost'] - $order->info['tax']),
            'currency_code' => $currency,
            'invoice' => substr($cart_PayPal_Standard_ID, strpos($cart_PayPal_Standard_ID, '-') + 1),
            'custom' => $customer_id,
            'no_note' => '1',
            'notify_url' => tep_href_link('ext/modules/payment/paypal/standard_ipn.php', '', 'SSL', false, false),
            'return' => $returnUrl,
            'cancel_return' => $cancelUrl,
            'bn' => 'osCommerce22_Default_ST',
            'paymentaction' => ((MODULE_PAYMENT_PAYPAL_STANDARD_TRANSACTION_METHOD == 'Sale') ? 'sale' : 'authorization'));

        if (is_numeric($sendto) && ($sendto > 0)) {
            $parameters['address_override'] = '1';
            $parameters['first_name'] = $order->delivery['firstname'];
            $parameters['last_name'] = $order->delivery['lastname'];
            $parameters['address1'] = $order->delivery['street_address'];
            $parameters['city'] = $order->delivery['city'];
            $parameters['state'] = tep_get_zone_code($order->delivery['country']['id'], $order->delivery['zone_id'], $order->delivery['state']);
            $parameters['zip'] = $order->delivery['postcode'];
            $parameters['country'] = $order->delivery['country']['iso_code_2'];
        } else {
            $parameters['no_shipping'] = '1';
            $parameters['first_name'] = $order->billing['firstname'];
            $parameters['last_name'] = $order->billing['lastname'];
            $parameters['address1'] = $order->billing['street_address'];
            $parameters['city'] = $order->billing['city'];
            $parameters['state'] = tep_get_zone_code($order->billing['country']['id'], $order->billing['zone_id'], $order->billing['state']);
            $parameters['zip'] = $order->billing['postcode'];
            $parameters['country'] = $order->billing['country']['iso_code_2'];
        }

        if (tep_not_null(MODULE_PAYMENT_PAYPAL_STANDARD_PAGE_STYLE)) {
            $parameters['page_style'] = MODULE_PAYMENT_PAYPAL_STANDARD_PAGE_STYLE;
        }

        if (MODULE_PAYMENT_PAYPAL_STANDARD_EWP_STATUS == 'True') {
            $parameters['cert_id'] = MODULE_PAYMENT_PAYPAL_STANDARD_EWP_CERT_ID;

            $random_string = rand(100000, 999999) . '-' . $customer_id . '-';

            $data = '';
            reset($parameters);
            while (list($key, $value) = each($parameters)) {
                $data .= $key . '=' . $value . "\n";
            }

            $fp = fopen(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'data.txt', 'w');
            fwrite($fp, $data);
            fclose($fp);

            unset($data);

            if (function_exists('openssl_pkcs7_sign') && function_exists('openssl_pkcs7_encrypt')) {
                openssl_pkcs7_sign(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'data.txt', MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt', file_get_contents(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PUBLIC_KEY), file_get_contents(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PRIVATE_KEY), array('From' => MODULE_PAYMENT_PAYPAL_STANDARD_ID), PKCS7_BINARY);

                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'data.txt');

                // remove headers from the signature
                $signed = file_get_contents(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt');
                $signed = explode("\n\n", $signed);
                $signed = base64_decode($signed[1]);

                $fp = fopen(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt', 'w');
                fwrite($fp, $signed);
                fclose($fp);

                unset($signed);

                openssl_pkcs7_encrypt(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt', MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt', file_get_contents(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PAYPAL_KEY), array('From' => MODULE_PAYMENT_PAYPAL_STANDARD_ID), PKCS7_BINARY);

                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt');

                // remove headers from the encrypted result
                $data = file_get_contents(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt');
                $data = explode("\n\n", $data);
                $data = '-----BEGIN PKCS7-----' . "\n" . $data[1] . "\n" . '-----END PKCS7-----';

                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt');
            } else {
                exec(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_OPENSSL . ' smime -sign -in ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'data.txt -signer ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PUBLIC_KEY . ' -inkey ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PRIVATE_KEY . ' -outform der -nodetach -binary > ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt');
                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'data.txt');

                exec(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_OPENSSL . ' smime -encrypt -des3 -binary -outform pem ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_PAYPAL_KEY . ' < ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt > ' . MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt');
                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'signed.txt');

                $fh = fopen(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt', 'rb');
                $data = fread($fh, filesize(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt'));
                fclose($fh);

                unlink(MODULE_PAYMENT_PAYPAL_STANDARD_EWP_WORKING_DIRECTORY . '/' . $random_string . 'encrypted.txt');
            }

            $process_button_string = tep_draw_hidden_field('cmd', '_s-xclick') .
                    tep_draw_hidden_field('encrypted', $data);

            unset($data);
        } else {
            reset($parameters);
            return $parameters;
        }
        return $parameters;
    }

    private function getSendToAddressId() {
        global $customer_default_address_id, $customer_id, $sendto;
        if (!tep_session_is_registered('sendto')) {
            tep_session_register('sendto');
            $sendto = $customer_default_address_id;
        } else {
            // verify the selected shipping address
            if ((is_array($sendto) && empty($sendto)) || is_numeric($sendto)) {
                $check_address_query = tep_db_query("select count(*) as total from " . TABLE_ADDRESS_BOOK . " where customers_id = '" . (int) $customer_id . "' and address_book_id = '" . (int) $sendto . "'");
                $check_address = tep_db_fetch_array($check_address_query);
                if ($check_address['total'] != '1') {
                    $sendto = $customer_default_address_id;
                    if (tep_session_is_registered('shipping'))
                        tep_session_unregister('shipping');
                }
            }
        }
        return $sendto;
    }

    public function getShippingAddress() {
        $sendTo = $this->getSendToAddressId();
        if ($sendTo) {
            $userService = ServiceFactory::factory('User');
            return $userService->getAddress($sendTo);
        }
        return array();
    }

    public function getBillingAddress() {
        $billto = $this->getBillToAddressId();
        if ($billto) {
            $userService = ServiceFactory::factory('User');
            return $userService->getAddress($billto);
        }
        return array();
    }

    public function getBillToAddressId() {
        global $billto, $customer_default_address_id, $customer_id;
        // if no billing destination address was selected, use the customers own address as default
        if (!tep_session_is_registered('billto')) {
            tep_session_register('billto');
            $billto = $customer_default_address_id;
        } else {
            // verify the selected billing address
            if ((is_array($billto) && empty($billto)) || is_numeric($billto)) {
                $check_address_query = tep_db_query("select count(*) as total from " . TABLE_ADDRESS_BOOK . " where customers_id = '" . (int) $customer_id . "' and address_book_id = '" . (int) $billto . "'");
                $check_address = tep_db_fetch_array($check_address_query);
                if ($check_address['total'] != '1') {
                    $billto = $customer_default_address_id;
                    if (tep_session_is_registered('payment'))
                        tep_session_unregister('payment');
                }
            }
        }
        return $billto;
    }

    public function getPaymentMethods() {

        $paypalwpp = array();
        $paypal = array();
        if (MODULE_PAYMENT_PAYPAL_EXPRESS_STATUS == 'True') {
            $paypalwpp['pm_id'] = 'paypalwpp';
            $paypalwpp['pm_title'] = '';
            $paypalwpp['pm_code'] = 'paypal_express';
            $paypalwpp['pm_description'] = '';
            $paypalwpp['img_url'] = '';
        }
        if (MODULE_PAYMENT_PAYPAL_STANDARD_STATUS == 'True') {
            $paypal['pm_id'] = 'paypal';
            $paypal['pm_title'] = '';
            $paypal['pm_code'] = 'paypal_standard';
            $paypal['pm_description'] = '';
            $paypal['img_url'] = '';
        }
        if ($paypal) {
            $paymentMethods[] = $paypal;
        }
        if ($paypalwpp) {
            $paymentMethods[] = $paypalwpp;
        }
        return $paymentMethods;
    }

    public function selectOrderShippingMethod() {
        
    }

    public function getOrderShippingMethods() {
        global $shipping, $currency, $order;
        if ($this->isVirtual) { //is virtual cart or not
            return array();
        }
        $shippingService = ServiceFactory::factory('Shipping');
        $shipping_modules = $shippingService->getShippingModules();
        $freeShipping = $this->getFreeShippingMethod($order); //is free shipping or not
        if (!is_null($freeShipping)) {
            return $freeShipping;
        }
        // get all available shipping quotes
        $quotes = $shipping_modules->quote();
        if (!$shipping || $shipping['id'] == 'free_free') {
            $shipping = $shipping_modules->cheapest();
        }
        $availableShippingMethods = array();
        for ($i = 0, $n = sizeof($quotes); $i < $n; $i++) {
            for ($j = 0, $n2 = sizeof($quotes[$i]['methods']); $j < $n2; $j++) {
                $shippingMethod = array();
                $shippingMethod['sm_id'] = $quotes[$i]['id'] . '_' . $quotes[$i]['methods'][$j]['id'];
                $shippingMethod['title'] = "{$quotes[$i]['module']}({$quotes[$i]['methods'][$j]['title']})";
                $shippingMethod['price'] = currency_price_value(tep_add_tax($quotes[$i]['methods'][$j]['cost'], (isset($quotes[$i]['tax']) ? $quotes[$i]['tax'] : 0)));
                $shippingMethod['currency'] = $currency;
                $shippingMethod['description'] = '';
                $availableShippingMethods[] = $shippingMethod;
            }
        }
        return $availableShippingMethods;
    }

    /**
     * get free shipping method,if not return null
     * @param type $order
     * @return string
     * @author hujs
     */
    public function getFreeShippingMethod($order) {
        global $currency, $language, $shipping;

        if (defined('MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING') && (MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING == 'true')) {
            $pass = false;

            switch (MODULE_ORDER_TOTAL_SHIPPING_DESTINATION) {
                case 'national':
                    if ($order->delivery['country_id'] == STORE_COUNTRY) {
                        $pass = true;
                    }
                    break;
                case 'international':
                    if ($order->delivery['country_id'] != STORE_COUNTRY) {
                        $pass = true;
                    }
                    break;
                case 'both':
                    $pass = true;
                    break;
            }
        }

        $freeShipping = ($pass == true) && ($order->info['subtotal'] >= MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING_OVER); //free shipping or not
        $this->freeShipping = $freeShipping;
        if ($freeShipping) {
            include_once(DIR_WS_LANGUAGES . $language . '/modules/order_total/ot_shipping.php');
            $shipping = array(
                'id' => 'free_free',
                'title' => FREE_SHIPPING_TITLE,
                'cost' => 0
            );
            $shippingMethod = array();
            $shippingMethod['sm_id'] = 'free_free';
            $shippingMethod['title'] = FREE_SHIPPING_TITLE;
            $shippingMethod['price'] = 0;
            $shippingMethod['currency'] = $currency;
            $shippingMethod['description'] = sprintf(FREE_SHIPPING_DESCRIPTION, MODULE_ORDER_TOTAL_SHIPPING_FREE_SHIPPING_OVER);

            return array($shippingMethod);
        }

        return null;
    }

    public function selectShippingMethod() {
        global $shipping;
        if (is_array($shipping)) {
            return $shipping['id'];
        } else if (is_string($shipping) && $shipping) {
            return $shipping;
        }
        return '';
    }

    public function getPriceInfos() {
        global $currency, $order_total_modules, $order;
        if (!class_exists('order')) {
            require(DIR_WS_CLASSES . 'order.php');
        }
        $order = new order;
        if (!class_exists('order_total')) {
            require(DIR_WS_CLASSES . 'order_total.php');
        }
        if (!$order_total_modules) {
            $order_total_modules = new order_total;
        }
        $totals = $order_total_modules->process();
        $priceInfo = array();
        $position = 0;
        foreach ($totals as $total) {
            $priceInfo[] = array(
                'title' => $total['title'],
                'type' => $total['code'] == 'ot_total' ? 'total' : $total['code'],
                'price' => currency_price_value($total['value']),
                'currency' => $currency,
                'position' => $position++
            );
        }
        return $priceInfo;
    }

    public function getOrderItems() {
        $shoppingCart = ServiceFactory::factory('ShoppingCart');
        $cartInfo = $shoppingCart->get();
        return $cartInfo['cart_items'];
    }

    public function addAddress($address) {
        global $sendto;
        $result = ServiceFactory::factory('User')->addAddress($address);
        if (is_numeric($result)) {
            $sendto = $result;
        }
        return $result;
    }

    public function updateAddress($addressBookId, $address = array()) {
        global $sendto;
        if ($addressBookId) {
            $sendto = $addressBookId;
            if ($address) {
                $address['address_book_id'] = $addressBookId;
                $userService = ServiceFactory::factory('User');
                $userService->updateAddress($address);
            }
        }
    }

    // update actions
    public function updateShippingMethod($shippingMethod) {
        global $free_shipping, $shipping, $order;
        $quote = array();
        $shippingService = ServiceFactory::factory('Shipping');
        $shipping_modules = $shippingService->getShippingModules();
        if ($shippingMethod) {
            $shipping = $shippingMethod;
            list($module, $method) = explode('_', $shippingMethod);
            global $$module;
            if (is_object($$module) || ($shipping == 'free_free')) {
                if ($shipping == 'free_free') {
                    $quote[0]['methods'][0]['title'] = FREE_SHIPPING_TITLE;
                    $quote[0]['methods'][0]['cost'] = '0';
                } else {
                    $quote = $shipping_modules->quote($method, $module);
                }
                if (isset($quote['error'])) {
                    tep_session_unregister('shipping');
                } else {
                    if ((isset($quote[0]['methods'][0]['title'])) && (isset($quote[0]['methods'][0]['cost']))) {
                        $shipping =
                                array(
                                    'id' => $shipping,
                                    'title' => (($free_shipping == true) ? $quote[0]['methods'][0]['title'] : $quote[0]['module'] . ' (' . $quote[0]['methods'][0]['title'] . ')'),
                                    'cost' => $quote[0]['methods'][0]['cost']
                        );
                        if ($order) {
                            if (class_exists('order')) {
                                // refresh the order
                                $order = new $order;
                            }
                        }
                    }
                }
            } else {
                tep_session_unregister('shipping');
            }
        }
    }

}

?>
