<?php

if (!defined('IN_KANCART')) {
    header('HTTP/1.1 404 Not Found');
    die();
}

/**
 * ShoppingCart Service,Utility
 * @package services 
 * @author hujs
 */
class ShoppingCartService {

    public function __construct() {
        $this->cart = &$_SESSION['cart'];
    }

    /**
     * Get Cart detailed information
     * @author hujs
     */
    public function get() {
        $result = array();
        $this->initShoppingCartGetReslut($result);

        if ($this->cart) {
            $result['cart_items'] = $this->getProducts();
            $result['price_infos'] = $this->getPriceInfo();
            $result['cart_items_count'] = $this->cart->count_contents();
            $result['is_virtual'] = $this->cart->content_type == 'virtual';  //get_content_type()
            if (MODULE_PAYMENT_PAYPAL_EXPRESS_STATUS == 'True') {
                $result['payment_methods'] = array('payapl_ec');
            }
        }

        return $result;
    }

    /**
     * get products information
     * @param type $cart
     * @return array
     * @author hujs
     */
    private function getProducts() {
        global $currency;

        $items = array();
        $productIds = array();  //product_id => cart_item_id
        $contents = $this->cart->contents;
        $productId = 0;

        if (is_array($contents)) {
            foreach (array_keys($contents) as $cartItemId) {
                $productId = $cartItemId;
                settype($productId, 'integer');
                $productIds[$productId][] = $cartItemId;
            }
        }

        $products = ServiceFactory::factory('Product')->getSpecifiedProducts(array_keys($productIds));

        foreach ($products['items'] as $product) {
            $hasDispalyPrices = isset($product['prices']) && isset($product['prices']['base_price']);
            $productId = $product['item_id'];

            //处理同名不同属性商品
            foreach ($productIds[$productId] as $cartItemId) {
                if ($hasDispalyPrices) {
                    $productPrice = $product['prices']['base_price']['price'] + currency_price_value(($this->cart->attributes_price($cartItemId)));
                } else {
                    $productPrice = 0;
                }
                $item = array(
                    'cart_item_id' => $cartItemId,
                    'cart_item_key' => '',
                    'item_id' => $productId,
                    'item_title' => $product['item_title'] . $this->getOutofStockMark($productId),
                    'thumbnail_pic_url' => $product['thumbnail_pic_url'],
                    'currency' => $currency,
                    'item_price' => $productPrice,
                    'item_original_price' => '',
                    'qty' => $contents[$cartItemId]['qty'],
                    'display_attributes' => $this->getProductAttributes($productId, $cartItemId),
                    'item_url' => $product['item_url'],
                    'short_description' => $product['short_description'],
                    'post_free' => false,
                );

                $items[] = $item;
            }
        }

        return $items;
    }

    private function getOutofStockMark($productId) {
        if (STOCK_CHECK == 'true') {
            foreach ($this->cart->contents as $cartItemId => $itemInfo) {
                $id = explode('{', $cartItemId);
                if ($id[0] == $productId) {
                    $stock_check = tep_check_stock($productId, $itemInfo['qty']);
                    if (tep_not_null($stock_check)) {
                        return STOCK_MARK_PRODUCT_OUT_OF_STOCK;
                    }
                }
            }
        }
        return '';
    }

    /**
     * get product display attributes in cart
     * @global type $languages_id
     * @param type $product_id $productIds
     * @return type
     * @author hujs
     */
    private function getProductAttributes($productId, $cartItemId) {
        global $languages_id;
        $productAttributes = '';
        $products = $this->cart->contents;

        if (isset($products[$cartItemId]['attributes']) && is_array($products[$cartItemId]['attributes'])) {
            while (list($option, $value) = each($products[$cartItemId]['attributes'])) {
                echo tep_draw_hidden_field('id[' . $products[$cartItemId]['id'] . '][' . $option . ']', $value);
                $attributesQuery = tep_db_query("select popt.products_options_name, poval.products_options_values_name
                                        from " . TABLE_PRODUCTS_OPTIONS . " popt, " . TABLE_PRODUCTS_OPTIONS_VALUES . " poval, " . TABLE_PRODUCTS_ATTRIBUTES . " pa
                                        where pa.products_id = '" . (int) $productId . "'
                                        and pa.options_id = '" . (int) $option . "'
                                        and pa.options_id = popt.products_options_id
                                        and pa.options_values_id = '" . (int) $value . "'
                                        and pa.options_values_id = poval.products_options_values_id
                                        and popt.language_id = '" . (int) $languages_id . "'
                                        and poval.language_id = '" . (int) $languages_id . "'");
                $row = tep_db_fetch_array($attributesQuery);
                $productAttributes .='<br>  - ' . $row['products_options_name'] . ' ' . $row['products_options_values_name'];
            }
        }

        return substr($productAttributes, 4);
    }

    /**
     * initialization cart information
     * @param type $result
     * @author hujs
     */
    public function initShoppingCartGetReslut(&$result) {
        $result['cart_items_count'] = 0;
        $result['cart_items'] = array();
        $result['messages'] = array();
        $result['price_infos'] = array();
        $result['payment_methods'] = array();
        $result['valid_to_checkout'] = true;
        $result['is_virtual'] = false;
    }

    /**
     * get price information
     * @return int
     * @author hujs
     */
    private function getPriceInfo() {
        global $currency, $language;

        $price_info = array();

        include(DIR_WS_LANGUAGES . $language . '/modules/order_total/ot_subtotal.php');

        $info = array();
        $info['title'] = MODULE_ORDER_TOTAL_SUBTOTAL_TITLE;
        $info['type'] = '';
        $info['price'] = currency_price_value($this->cart->show_total());
        $info['home_currency_price'] = currency_price_value($this->cart->total);
        $info['currency'] = $currency;
        $info['position'] = 1;

        $price_info[] = $info;

        return $price_info;
    }

    /**
     * add goods into cart
     * @param type $goods
     * @return type 
     * @author hujs
     */
    public function add($products) {
        $this->clearShipping();
        return $this->cart->add_cart($products['products_id'], $products['qty'], $products['attr'], true);
    }

    public function clearShipping() {
        if (tep_session_is_registered('shipping')) {
            tep_session_unregister('shipping');
        }
    }

    /**
     * update product's quantity and attributes
     * in cart by product id
     * @param type $arr
     * @return type 
     * @author hujs
     */
    public function update($productsId, $quantity = '', $attributes = '') {
        if (empty($attributes)) {
            $contents = $this->cart->contents;
            $attributes = $contents[$productsId]['attributes'];
        }
        $this->clearShipping();
        return $this->cart->update_quantity($productsId, $quantity, $attributes);
    }

    /**
     * remove goods from cart by product id
     * @access  public
     * @param   integer $id
     * @return  void
     * @author hujs
     */
    public function dropCartGoods($productId) {
        $this->clearShipping();
        $this->cart->remove($productId);
    }

    /**
     * get a product quantity
     * @param type $productId
     * @return  int
     * @author hujs
     */
    public function getProductQuantity($productId) {

        $productQuery = tep_db_query("SELECT products_quantity FROM " . TABLE_PRODUCTS . " WHERE products_id = " . (int) $productId);
        $row = tep_db_fetch_array($productQuery);

        return (int) $row['products_quantity'];
    }

    /**
     * Determine whether the merchandise is virtual goods
     * @param type $productId
     * @return boolean
     * @author hujs
     */
    public function isVirtualProduct($productId) {

        if (empty($productId)) {
            return false;
        }
        settype(&$productId, 'integer');

        if ($productId <= 0) {
            return false;
        }

        $virtualQuery = tep_db_query("select count(*) as count from " . TABLE_PRODUCTS_ATTRIBUTES . " pa, " . TABLE_PRODUCTS_ATTRIBUTES_DOWNLOAD . " pad where pa.products_id = '" . (int) $productId . "' and pa.products_attributes_id = pad.products_attributes_id");
        $result = tep_db_fetch_array($virtualQuery);

        return $result['count'] > 0;
    }

}

?>
