<?php

if (!defined('IN_KANCART')) {
    header('HTTP/1.1 404 Not Found');
    die();
}

class StoreService {

    public function getStoreInfo() {
        $storeInfo = array();
        $storeInfo['general'] = $this->getGeneralInfo();
        $storeInfo['currencies'] = $this->getCurrencies();
        $storeInfo['countries'] = $this->getCountries();
        $storeInfo['zones'] = $this->getZones();
        $storeInfo['languages'] = $this->getLanguages();
        //$storeInfo['order_statuses'] = $this->getOrderStatauses();
        $storeInfo['register_fields'] = $this->getRegisterFields();
        $storeInfo['address_fields'] = $this->getAddressFields();
        $storeInfo['category_sort_options'] = $this->getCategorySortOptions();
        $storeInfo['search_sort_options'] = $this->getSearchSortOptions();
        return $storeInfo;
    }

    /**
     * get Languages
     * @global type $languages
     * @return string
     * @author hujs
     */
    public function getLanguages() {
        $info = array();
        $position = 0;
        $languages_query = tep_db_query("select languages_id, name, code, image, directory from " . TABLE_LANGUAGES . " order by sort_order");
        while ($languages = tep_db_fetch_array($languages_query)) {
            $info[] = array(
                'language_id' => $languages['languages_id'],
                'default' => $languages['code'] == DEFAULT_LANGUAGE,
                'language_code' => $languages['code'],
                'language_name' => $languages['name'],
                'language_text' => $languages['directory'],
                'position' => $position++,
            );
        }
        return $info;
    }

    public function getCountries() {
        $shopCountries = array();
        $countries = tep_db_query("select * from " . TABLE_COUNTRIES . " order by countries_name");
        while ($row = tep_db_fetch_array($countries)) {
            $shopCountries[] = array(
                'country_id' => $row['countries_id'],
                'country_name' => $row['countries_name'],
                'country_iso_code_2' => $row['countries_iso_code_2'],
                'country_iso_code_3' => $row['countries_iso_code_3']
            );
        }
        return $shopCountries;
    }

    public function getZones() {
        $shopZones = array();
        $query = tep_db_query("select * from " . TABLE_ZONES);
        while ($row = tep_db_fetch_array($query)) {
            $shopZones[] = array(
                'zone_id' => $row['zone_id'],
                'country_id' => $row['zone_country_id'],
                'zone_name' => $row['zone_name'],
                'zone_code' => $row['zone_code']
            );
        }
        return $shopZones;
    }

    public function getCurrencies() {
        $currencyService = ServiceFactory::factory('Currency');
        return $currencyService->getCurrencies();
    }

    public function getOrderStatauses() {
        $_LANG = $GLOBALS['_LANG'];
        $orderStatuses = array(
            array(
                'status_id' => OS_UNCONFIRMED,
                'status_name' => $_LANG['os'][OS_UNCONFIRMED],
            ),
            array(
                'status_id' => OS_CONFIRMED,
                'status_name' => $_LANG['os'][OS_CONFIRMED],
            ),
            array(
                'status_id' => OS_CANCELED,
                'status_name' => $_LANG['os'][OS_CANCELED],
            ),
            array(
                'status_id' => OS_INVALID,
                'status_name' => $_LANG['os'][OS_INVALID],
            ),
            array(
                'status_id' => OS_RETURNED,
                'status_name' => $_LANG['os'][OS_RETURNED],
            ),
            array(
                'status_id' => OS_SPLITED,
                'status_name' => $_LANG['os'][OS_SPLITED],
            ),
            array(
                'status_id' => OS_SPLITING_PART,
                'status_name' => $_LANG['os'][OS_SPLITING_PART],
            ),
        );
        return $orderStatuses;
    }

    public function getGeneralInfo() {
        return array(
            'cart_type' => 'oscommerce',
            'cart_version' => tep_get_version(),
            'login_by_mail' => true
        );
    }

    public function getRegisterFields() {
        $registerFields = array(
            array('type' => 'firstname', 'required' => true),
            array('type' => 'lastname', 'required' => true),
            array('type' => 'email', 'required' => true),
            array('type' => 'pwd', 'required' => true),
            array('type' => 'telephone', 'required' => true)
        );
        return $registerFields;
    }

    public function getAddressFields() {
        $addressFields = array(
            array('type' => 'firstname', 'required' => true),
            array('type' => 'lastname', 'required' => true),
            array('type' => 'gender', 'required' => true),
            array('type' => 'country', 'required' => true),
            array('type' => 'zone', 'required' => true),
            array('type' => 'city', 'required' => true),
            array('type' => 'address1', 'required' => true),
            array('type' => 'address2', 'required' => false),
            array('type' => 'postcode', 'required' => true),
        );
        return $addressFields;
    }

    public function getCategorySortOptions() {
        global $language;
        require_once(DIR_WS_LANGUAGES . $language . '/' . FILENAME_DEFAULT);
        $define_list = array(
            'PRODUCT_LIST_MODEL' => PRODUCT_LIST_MODEL,
            'PRODUCT_LIST_NAME' => PRODUCT_LIST_NAME,
            'PRODUCT_LIST_MANUFACTURER' => PRODUCT_LIST_MANUFACTURER,
            'PRODUCT_LIST_PRICE' => PRODUCT_LIST_PRICE,
            'PRODUCT_LIST_QUANTITY' => PRODUCT_LIST_QUANTITY,
            'PRODUCT_LIST_WEIGHT' => PRODUCT_LIST_WEIGHT);

        asort($define_list);
        $column_list = array();
        reset($define_list);
        while (list($key, $value) = each($define_list)) {
            if ($value > 0)
                $column_list[] = $key;
        }
        $categorySortOptions = array();
        for ($col = 0, $n = sizeof($column_list); $col < $n; $col++) {
            switch ($column_list[$col]) {
                case 'PRODUCT_LIST_MODEL': {
                        $categorySortOptions[] = array(
                            array(
                                'title' => TABLE_HEADING_MODEL,
                                'code' => 'p.products_model:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_MODEL,
                                'code' => 'p.products_model:desc',
                                'arrow_type' => 'desc'
                            )
                        );
                    }
                    break;
                case 'PRODUCT_LIST_NAME': {
                        $categorySortOptions[] = array(array(
                                'title' => TABLE_HEADING_PRODUCTS,
                                'code' => 'pd.products_name:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_PRODUCTS,
                                'code' => 'pd.products_name:desc',
                                'arrow_type' => 'desc'
                                ));
                    }
                    break;
                case 'PRODUCT_LIST_MANUFACTURER': {
                        $categorySortOptions[] = array(
                            array(
                                'title' => TABLE_HEADING_MANUFACTURER,
                                'code' => 'm.manufacturers_name:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_MANUFACTURER,
                                'code' => 'm.manufacturers_name:desc',
                                'arrow_type' => 'desc'
                            )
                        );
                    }
                    break;
                case 'PRODUCT_LIST_PRICE': {
                        $categorySortOptions[] = array(
                            array(
                                'title' => TABLE_HEADING_PRICE,
                                'code' => 'final_price:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_PRICE,
                                'code' => 'final_price:desc',
                                'arrow_type' => 'desc'
                            )
                        );
                    }
                    break;
                case 'PRODUCT_LIST_QUANTITY': {
                        $categorySortOptions[] = array(
                            array(
                                'title' => TABLE_HEADING_QUANTITY,
                                'code' => 'p.products_quantity:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_QUANTITY,
                                'code' => 'p.products_quantity:desc',
                                'arrow_type' => 'desc'
                            )
                        );
                    }
                    break;
                case 'PRODUCT_LIST_WEIGHT': {
                        $categorySortOptions[] = array(
                            array(
                                'title' => TABLE_HEADING_WEIGHT,
                                'code' => 'p.products_weight:asc',
                                'arrow_type' => 'asc'
                            ),
                            array(
                                'title' => TABLE_HEADING_WEIGHT,
                                'code' => 'p.products_weight:desc',
                                'arrow_type' => 'desc'
                            )
                        );
                    }
                    break;
            }
        }
        return $categorySortOptions;
    }

    public function getSearchSortOptions() {
        return $this->getCategorySortOptions();
    }

}

?>
